@@ -183,8 +183,8 @@ int main(int argc, char **argv)
 		printf("    -b backend\n");
 		printf("        use this backend for the output file specified on the command line\n");
 		printf("\n");
-		printf("    -f backend\n");
-		printf("        use the specified front for the input files on the command line\n");
+		printf("    -f frontend\n");
+		printf("        use the specified frontend for the input files on the command line\n");
 		printf("\n");
 		printf("    -H\n");
 		printf("        print the command list\n");
@@ -213,6 +213,11 @@ int main(int argc, char **argv)
 		printf("    -A\n");
 		printf("        will call abort() at the end of the script. for debugging\n");
 		printf("\n");
+		printf("    -D <header_id>[:<filename>]\n");
+		printf("        dump the design when printing the specified log header to a file.\n");
+		printf("        yosys_dump_<header_id>.il is used as filename if none is specified.\n");
+		printf("        Use 'ALL' as <header_id> to dump at every header.\n");
+		printf("\n");
 		printf("    -V\n");
 		printf("        print version information and exit\n");
 		printf("\n");
@@ -233,7 +238,7 @@ int main(int argc, char **argv)
 	}
 
 	int opt;
-	while ((opt = getopt(argc, argv, "MXAQTVSm:f:Hh:b:o:p:l:L:qv:tds:c:")) != -1)
+	while ((opt = getopt(argc, argv, "MXAQTVSm:f:Hh:b:o:p:l:L:qv:tds:c:D:")) != -1)
 	{
 		switch (opt)
 		{
@@ -315,6 +320,28 @@ int main(int argc, char **argv)
 			scriptfile = optarg;
 			scriptfile_tcl = true;
 			break;
+		case 'D':
+			{
+				auto args = split_tokens(optarg, ":");
+				if (!args.empty() && args[0] == "ALL") {
+					if (GetSize(args) != 1) {
+						fprintf(stderr, "Invalid number of tokens in -D ALL.\n");
+						exit(1);
+					}
+					log_hdump_all = true;
+				} else {
+					if (!args.empty() && !args[0].empty() && args[0].back() == '.')
+						args[0].pop_back();
+					if (GetSize(args) == 1)
+						args.push_back("yosys_dump_" + args[0] + ".il");
+					if (GetSize(args) != 2) {
+						fprintf(stderr, "Invalid number of tokens in -D.\n");
+						exit(1);
+					}
+					log_hdump[args[0]].insert(args[1]);
+				}
+			}
+			break;
 		default:
 			fprintf(stderr, "Run '%s -h' for help.\n", argv[0]);
 			exit(1);
@@ -482,6 +509,13 @@ int main(int argc, char **argv)
 		free(hist_list);
 #endif
 
+	log_flush();
+#if defined(_MSC_VER)
+	_exit(0);
+#elif defined(_WIN32)
+	_Exit(0);
+#endif
+
 	yosys_shutdown();
 
 	return 0;
