@@ -80,6 +80,7 @@ Pass::pre_post_exec_state_t Pass::pre_execute()
 	state.begin_ns = PerformanceTimer::query();
 	state.parent_pass = current_pass;
 	current_pass = this;
+	clear_flags();
 	return state;
 }
 
@@ -99,6 +100,10 @@ void Pass::help()
 	log("\n");
 }
 
+void Pass::clear_flags()
+{
+}
+
 void Pass::cmd_log_args(const std::vector<std::string> &args)
 {
 	if (args.size() <= 1)
@@ -160,7 +165,7 @@ void Pass::call(RTLIL::Design *design, std::string command)
 		while (!cmd_buf.empty() && (cmd_buf.back() == ' ' || cmd_buf.back() == '\t' ||
 				cmd_buf.back() == '\r' || cmd_buf.back() == '\n'))
 			cmd_buf.resize(cmd_buf.size()-1);
-		log_header("Shell command: %s\n", cmd_buf.c_str());
+		log_header(design, "Shell command: %s\n", cmd_buf.c_str());
 		int retCode = run_command(cmd_buf);
 		if (retCode != 0)
 			log_cmd_error("Shell command returned error code %d.\n", retCode);
@@ -282,6 +287,60 @@ void Pass::call_on_module(RTLIL::Design *design, RTLIL::Module *module, std::vec
 	design->selected_active_module = backup_selected_active_module;
 }
 
+bool ScriptPass::check_label(std::string label, std::string info)
+{
+	if (active_design == nullptr) {
+		log("\n");
+		if (info.empty())
+			log("    %s:\n", label.c_str());
+		else
+			log("    %s:    %s\n", label.c_str(), info.c_str());
+		return true;
+	} else {
+		if (!active_run_from.empty() && active_run_from == active_run_to) {
+			block_active = (label == active_run_from);
+		} else {
+			if (label == active_run_from)
+				block_active = true;
+			if (label == active_run_to)
+				block_active = false;
+		}
+		return block_active;
+	}
+}
+
+void ScriptPass::run(std::string command, std::string info)
+{
+	if (active_design == nullptr) {
+		if (info.empty())
+			log("        %s\n", command.c_str());
+		else
+			log("        %s    %s\n", command.c_str(), info.c_str());
+	} else
+		Pass::call(active_design, command);
+}
+
+void ScriptPass::run_script(RTLIL::Design *design, std::string run_from, std::string run_to)
+{
+	help_mode = false;
+	active_design = design;
+	block_active = run_from.empty();
+	active_run_from = run_from;
+	active_run_to = run_to;
+	script();
+}
+
+void ScriptPass::help_script()
+{
+	clear_flags();
+	help_mode = true;
+	active_design = nullptr;
+	block_active = true;
+	active_run_from.clear();
+	active_run_to.clear();
+	script();
+}
+
 Frontend::Frontend(std::string name, std::string short_help) :
 		Pass(name.rfind("=", 0) == 0 ? name.substr(1) : "read_" + name, short_help),
 		frontend_name(name.rfind("=", 0) == 0 ? name.substr(1) : name)
@@ -323,7 +382,8 @@ void Frontend::extra_args(std::istream *&f, std::string &filename, std::vector<s
 	bool called_with_fp = f != NULL;
 
 	next_args.clear();
-	for (; argidx < args.size(); argidx++)
+
+	if (argidx < args.size())
 	{
 		std::string arg = args[argidx];
 
@@ -360,6 +420,12 @@ void Frontend::extra_args(std::istream *&f, std::string &filename, std::vector<s
 			f = new std::istringstream(last_here_document);
 		} else {
 			rewrite_filename(filename);
+			vector<string> filenames = glob_filename(filename);
+			filename = filenames.front();
+			if (GetSize(filenames) > 1) {
+				next_args.insert(next_args.end(), args.begin(), args.begin()+argidx);
+				next_args.insert(next_args.end(), filenames.begin()+1, filenames.end());
+			}
 			std::ifstream *ff = new std::ifstream;
 			ff->open(filename.c_str());
 			if (ff->fail())
@@ -375,12 +441,13 @@ void Frontend::extra_args(std::istream *&f, std::string &filename, std::vector<s
 				cmd_error(args, i, "Found option, expected arguments.");
 
 		if (argidx+1 < args.size()) {
-			next_args.insert(next_args.begin(), args.begin(), args.begin()+argidx);
-			next_args.insert(next_args.begin()+argidx, args.begin()+argidx+1, args.end());
+			if (next_args.empty())
+				next_args.insert(next_args.end(), args.begin(), args.begin()+argidx);
+			next_args.insert(next_args.end(), args.begin()+argidx+1, args.end());
 			args.erase(args.begin()+argidx+1, args.end());
 		}
-		break;
 	}
+
 	if (f == NULL)
 		cmd_error(args, argidx, "No filename given.");
 
