@@ -51,6 +51,7 @@
 #include <initializer_list>
 #include <stdexcept>
 #include <memory>
+#include <cmath>
 
 #include <sstream>
 #include <fstream>
@@ -63,6 +64,7 @@
 #include <string.h>
 #include <stdint.h>
 #include <stdio.h>
+#include <limits.h>
 
 #ifndef _YOSYS_
 #  error It looks like you are trying to build Yosys without the config defines set. \
@@ -91,14 +93,18 @@
 #  define mkdir _mkdir
 #  define popen _popen
 #  define pclose _pclose
-#  define PATH_MAX MAX_PATH
 
 #  ifndef __MINGW32__
+#    define PATH_MAX MAX_PATH
 #    define isatty _isatty
 #    define fileno _fileno
 #  endif
 #endif
 
+#ifndef PATH_MAX
+#  define PATH_MAX 4096
+#endif
+
 #define PRIVATE_NAMESPACE_BEGIN  namespace {
 #define PRIVATE_NAMESPACE_END    }
 #define YOSYS_NAMESPACE_BEGIN    namespace Yosys {
@@ -279,6 +285,7 @@ RTLIL::Design *yosys_get_design();
 std::string proc_self_dirname();
 std::string proc_share_dirname();
 const char *create_prompt(RTLIL::Design *design, int recursion_counter);
+std::vector<std::string> glob_filename(const std::string &filename_pattern);
 void rewrite_filename(std::string &filename);
 
 void run_pass(std::string command, RTLIL::Design *design = nullptr);
