@@ -582,8 +582,9 @@ struct ShowPass : public Pass {
 		log("        Run the specified command with the graphics file as parameter.\n");
 		log("\n");
 		log("    -format <format>\n");
-		log("        Generate a graphics file in the specified format.\n");
-		log("        Usually <format> is 'svg' or 'ps'.\n");
+		log("        Generate a graphics file in the specified format. Use 'dot' to just\n");
+		log("        generate a .dot file, or other <format> strings such as 'svg' or 'ps'\n");
+		log("        to generate files in other formats (this calls the 'dot' command).\n");
 		log("\n");
 		log("    -lib <verilog_or_ilang_file>\n");
 		log("        Use the specified library file for determining whether cell ports are\n");
@@ -646,12 +647,13 @@ struct ShowPass : public Pass {
 		log("unless another prefix is specified using -prefix <prefix>.\n");
 		log("\n");
 		log("Yosys on Windows and YosysJS use different defaults: The output is written\n");
-		log("to 'show.dot' in the current directory and new viewer is launched.\n");
+		log("to 'show.dot' in the current directory and new viewer is launched each time\n");
+		log("the 'show' command is executed.\n");
 		log("\n");
 	}
 	virtual void execute(std::vector<std::string> args, RTLIL::Design *design)
 	{
-		log_header("Generating Graphviz representation of design.\n");
+		log_header(design, "Generating Graphviz representation of design.\n");
 		log_push();
 
 		std::vector<std::pair<std::string, RTLIL::Selection>> color_selections;
@@ -759,7 +761,7 @@ struct ShowPass : public Pass {
 		}
 		extra_args(args, argidx, design);
 
-		if (format != "ps") {
+		if (format != "ps" && format != "dot") {
 			int modcount = 0;
 			for (auto &mod_it : design->modules_) {
 				if (mod_it.second->get_bool_attribute("\\blackbox"))
@@ -770,7 +772,7 @@ struct ShowPass : public Pass {
 					modcount++;
 			}
 			if (modcount > 1)
-				log_cmd_error("For formats different than 'ps' only one module must be selected.\n");
+				log_cmd_error("For formats different than 'ps' or 'dot' only one module must be selected.\n");
 		}
 
 		for (auto filename : libfiles) {
@@ -784,7 +786,7 @@ struct ShowPass : public Pass {
 		}
 
 		if (libs.size() > 0)
-			log_header("Continuing show pass.\n");
+			log_header(design, "Continuing show pass.\n");
 
 		std::string dot_file = stringf("%s.dot", prefix.c_str());
 		std::string out_file = stringf("%s.%s", prefix.c_str(), format.empty() ? "svg" : format.c_str());
@@ -806,7 +808,7 @@ struct ShowPass : public Pass {
 			log_cmd_error("Nothing there to show.\n");
 
 		if (format != "dot" && !format.empty()) {
-			std::string cmd = stringf("dot -T%s -o '%s.new' '%s' && mv '%s.new' '%s'", format.c_str(), out_file.c_str(), dot_file.c_str(), out_file.c_str(), out_file.c_str());
+			std::string cmd = stringf("dot -T%s '%s' > '%s.new' && mv '%s.new' '%s'", format.c_str(), dot_file.c_str(), out_file.c_str(), out_file.c_str(), out_file.c_str());
 			log("Exec: %s\n", cmd.c_str());
 			if (run_command(cmd) != 0)
 				log_cmd_error("Shell command failed!\n");
