@@ -198,7 +198,7 @@ struct EquivInductPass : public Pass {
 		bool model_undef = false;
 		int max_seq = 4;
 
-		log_header("Executing EQUIV_INDUCT pass.\n");
+		log_header(design, "Executing EQUIV_INDUCT pass.\n");
 
 		size_t argidx;
 		for (argidx = 1; argidx < args.size(); argidx++) {
