@@ -656,6 +656,9 @@ grow_read_ports:;
 		bool transp = rd_transp[cell_port_i] == State::S1;
 		SigBit clksig = rd_clk[cell_port_i];
 
+		if (wr_ports == 0)
+			transp = false;
+
 		pair<SigBit, bool> clkdom(clksig, clkpol);
 		if (!clken)
 			clkdom = pair<SigBit, bool>(State::S1, false);
@@ -826,7 +829,7 @@ grow_read_ports:;
 					State padding = State::Sx;
 					for (int j = 0; j < bram.dbits; j++)
 						if (init_offset+i < GetSize(initdata) && init_shift+j < GetSize(initdata[init_offset+i]))
-							padding = initparam[i*bram.dbits+j] = initdata[init_offset+i][init_shift+j];
+							initparam[i*bram.dbits+j] = initdata[init_offset+i][init_shift+j];
 						else
 							initparam[i*bram.dbits+j] = padding;
 				}
@@ -1211,7 +1214,7 @@ struct MemoryBramPass : public Pass {
 	{
 		rules_t rules;
 
-		log_header("Executing MEMORY_BRAM pass (mapping $mem cells to block memories).\n");
+		log_header(design, "Executing MEMORY_BRAM pass (mapping $mem cells to block memories).\n");
 
 		size_t argidx;
 		for (argidx = 1; argidx < args.size(); argidx++) {
